#
#   Imandra Inc.
#
#   speclogician/demos/model.py
#

from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path

@dataclass(frozen=True)
class DemoSpec:
    name: str
    root: Path

    @property
    def artifacts_path(self) -> Path:
        return self.root / "artifacts.md"

    @property
    def description_path(self) -> Path:
        return self.root / "description.md"

    @property
    def changes_path(self) -> Path:
        return self.root / "changes.py"