#
#   Imandra Inc.
#
#   speclogician/data/refs.py
#

from .artifact import DataArtifact

class DocRef(DataArtifact):
    """
    Documentation reference
    """
    text : str # Actual documentation text
    meta : str | None = None # Any additional information
    
class SrcCodeRef(DataArtifact):
    """
    Source code reference
    """
    src_code : str # The original source code
    language : str | None  = None # Programming language of the source code
    file_path : str | None = None # File path of the source file
    iml_code : str | None = None # Formalized version of the source code in IML
    meta : str | None = None # Any additional information about the source code reference