#
#   Imandra Inc.
#
#   speclogician/state/recommendation.py
#

from pydantic import BaseModel
from typing import Literal

class Recommendation(BaseModel):
    text: str
    kind: Literal["error", "warning", "next", "info"] = "next"
    priority: int = 50  # lower = higher priority
